package com.DUNS;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.FileDescriptor;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.bluetooth.BluetoothAdapter;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager.NameNotFoundException;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.widget.Toast;

public class DUNService extends Service {
	static {
        System.loadLibrary("DUNS");
    }
	public static native void closeptymx(FileDescriptor fd);
	public static native int ptymx(FileDescriptor fd);
	public static native FileDescriptor openptymx();
	
	private NotificationManager mNM;
	
	@Override
	public IBinder onBind(Intent arg0) {
		return null;
	}

    // Message types sent from the BluetoothBTService Handler
    public static final int MESSAGE_STATE_CHANGE = 1;
    public static final int MESSAGE_READ = 2;
    public static final int MESSAGE_WRITE = 3;
    public static final int MESSAGE_DEVICE_NAME = 4;
    public static final int MESSAGE_TOAST = 5;

    // Key names received from the BluetoothBTService Handler
    public static final String DEVICE_NAME = "device_name";
    public static final String TOAST = "toast";

    public static boolean ServiceRunning = false;

    private PPPThread PPP;
    private Process pppprocess = null;
    private String pppptyname;
    private BufferedInputStream pppin = null;
    private FileOutputStream pppout = null;
    private FileDescriptor fd = null;
	private int pts;
    private Notification notification = null;
    PendingIntent contentIntent = null;

    // Name of the connected device
    private String mConnectedDeviceName = null;
    // Local Bluetooth adapter
    private BluetoothAdapter mBluetoothAdapter = null;
    // Member object for the chat services
    private BluetoothService mBTService = null;

    public void clearNotifications() {
    	mNM.cancelAll();
    }

    @Override
    public void onCreate() {
        ServiceRunning = true;

		mNM = (NotificationManager)getSystemService(NOTIFICATION_SERVICE);

        // Get local Bluetooth adapter
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();

        // If the adapter is null, then Bluetooth is not supported
        if (mBluetoothAdapter == null) {
            Toast.makeText(this, "Bluetooth is not available", Toast.LENGTH_LONG).show();
            stopSelf();
            return;
        }
        
        try {
        	FileOutputStream os;
        	InputStream is;
        	int size;
        	int ver = 9999999;
            byte[] buffer;
            FileInputStream fis = null;
            BufferedReader in = null;
        
            try{
               fis = new FileInputStream("setup.sh");
               in = new BufferedReader(new InputStreamReader(fis));
         
               String currentLine = "";
               while ((currentLine = in.readLine()) != null) {
                  if(currentLine.indexOf("# Version: ") > 0)  {
                	  if (Character.isDigit(currentLine.charAt(11))) {
                		  ver = Integer.valueOf(currentLine.substring(11));
                	  }
                  }
               }
         
            }catch(IOException ioe){
               ioe.printStackTrace();
            }finally{
               try{
                  if(in != null) in.close();
                  if(fis != null) fis.close();
               }catch(IOException ioe){ }
            }
            if(ver< getPackageManager().getPackageInfo(getPackageName(), 0).versionCode) {
            	is = getAssets().open("setup.sh");
            	size = is.available();
            	buffer = new byte[size];
            	is.read(buffer);
            	is.close();

            	os =  openFileOutput("setup.sh", Context.MODE_PRIVATE);
            	os.write(buffer);
            	os.close();
            }
            
        	Process p = Runtime.getRuntime().exec("/system/bin/chmod 0755 /data/data/com.DUNS/files/setup.sh");
        	p.waitFor();
        	p =Runtime.getRuntime().exec("/data/data/com.DUNS/files/setup.sh");
        	p.waitFor();
        } catch (IOException e) {
		} catch (InterruptedException e) {
		} catch (NameNotFoundException e) {
		}
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        if (mBTService == null) 
        	setupBT();

        notification = new Notification(R.drawable.app_icon, "BT DUN Service", System.currentTimeMillis());   
        notification.flags |= Notification.FLAG_AUTO_CANCEL;
        
        contentIntent = PendingIntent.getActivity(this, 0,
                    new Intent(this, DUNIntent.class), 0);

        notification.setLatestEventInfo(this, "BT DUN Service",
        		"Starting", contentIntent);
            
        startForeground(1,notification);
        mNM.notify(1,notification);
        return START_STICKY | START_FLAG_REDELIVERY;
    }

    private void setupBT() {
        mBTService = new BluetoothService(this, mHandler);
        mBTService.start();
    }

    private void closefiles() {
    	try {
    		if(PPP!=null) {
    			PPP.interrupt();
    			PPP = null;
    		}
    		if(pppprocess != null) {
    			pppprocess.destroy();
    			pppprocess = null;
    		}
    		if(pppin != null) {
    			pppin.close();
    			pppin = null;
    		}
    		if(pppout != null) {
    			pppout.close();
    			pppout = null;
    		}
    		if(fd != null) {
    			closeptymx(fd);
    			fd = null;
    		}
    	} catch(IOException e) {
    	}
    }
    
    @Override
    public void onDestroy() {
    	ServiceRunning = false;
        // Stop the Bluetooth chat services
        if (mBTService != null) mBTService.stop();
        mBTService = null;
        closefiles();
        mNM.cancel(1);
    }

    private synchronized void sendMessage(String message) {
        // Check that there's actually something to send
        if (message.length() > 0) {
            // Get the message bytes and tell the BluetoothBTService to write
            byte[] send = message.getBytes();
            mBTService.write(send);
        }
    }
    
    private synchronized void sendMessage(byte[] message, int len) {
        // Check that there's actually something to send
        if (message.length > 0) {
            // Get the message bytes and tell the BluetoothBTService to write
            mBTService.write(message, len);
        }
    }

    private void notify(String text, int icon) {
	    mNM.cancel(1);
        notification.setLatestEventInfo(DUNService.this, "BT DUN Service",
        		text, contentIntent);
        notification.icon = icon;
        mNM.notify(1,notification);

    }
    
    private class PPPThread extends Thread {
        PPPThread() {
        }

        public void run() {
        	try {
        	    int rd;
        	    byte[] buffer = new byte[4096];
        	    while (true) {
        	    	if ((rd = pppin.read(buffer, 0, 1600)) > 0) {
        	    		sendMessage(buffer, rd);
        	    	} else
        	    	Thread.sleep(1);
        	    }
        	} catch (IOException e) {
        		closefiles();
        		mBTService.start();
        	} catch (InterruptedException e) {
        		closefiles();
        		mBTService.start();
			}               	

        }
    }

    // The Handler that gets information back from the BluetoothService
    private final Handler mHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
            case MESSAGE_STATE_CHANGE:
                switch (msg.arg1) {
                case BluetoothService.STATE_CONNECTED:
                    DUNService.this.notify("Connected to "+ mConnectedDeviceName, R.drawable.bt_conn);
//                    Toast.makeText(DUNService.this, "Connected to "+ mConnectedDeviceName, Toast.LENGTH_SHORT).show();
                    break;
                case BluetoothService.STATE_LISTEN:
                case BluetoothService.STATE_NONE:
                	closefiles();
                    DUNService.this.notify("Not connected.", R.drawable.bt_ready);
                    break;
                }
                break;
            case MESSAGE_WRITE:
                break;
            case MESSAGE_READ:
                byte[] readBuf = (byte[]) msg.obj;
                
                if(pppout!=null) {
                	try {
                		if(msg.arg1>0) pppout.write(readBuf, 0, msg.arg1);
					} catch (IOException e) {
					}
                } else {
                    // construct a string from the valid bytes in the buffer
                	String readMessage = new String(readBuf, 0, msg.arg1);

                	if(readMessage.startsWith("ATDT")) {
                		DUNService.this.sendMessage("CONNECT\r");
                    	try {
                  			fd = openptymx();
                   			pts = ptymx(fd);
                			pppin = new BufferedInputStream(new FileInputStream(fd));
                			pppout = new FileOutputStream(fd);
                    		pppprocess = Runtime.getRuntime().exec("/system/bin/pppd /dev/pts/" + Integer.toString(pts) + " local nodetach passive persist 10.123.214.6:10.123.214.7");

                            DUNService.this.notify("PPP started "+ mConnectedDeviceName, R.drawable.bt_ppp);
                    	        	    
                    	} catch (IOException e) {
                    		closefiles();
                		}

                		PPP = new PPPThread();
                		PPP.start();

                	} else
                		if(readMessage.startsWith("AT")) DUNService.this.sendMessage("OK\r");
                }
                break;
            case MESSAGE_DEVICE_NAME:
                // save the connected device's name
                mConnectedDeviceName = msg.getData().getString(DEVICE_NAME);
                DUNService.this.notify("Connected to "+ mConnectedDeviceName, R.drawable.bt_conn);
                break;
            case MESSAGE_TOAST:
                Toast.makeText(getApplicationContext(), msg.getData().getString(TOAST),
                               Toast.LENGTH_SHORT).show();
                break;
            }
        }
    };
}
